// © 2021 Dag Langmyhr, Institutt for informatikk, Universitetet i Oslo

package no.uio.ifi.asp.parser;    

import java.util.ArrayList;

import no.uio.ifi.asp.main.*;
import no.uio.ifi.asp.runtime.*;
import no.uio.ifi.asp.scanner.*;
import static no.uio.ifi.asp.scanner.TokenKind.*;

class AspArguments extends AspPrimarySuffix {
    ArrayList<AspExpr> args = new ArrayList<>();

    AspArguments(int n) {
	super(n);
    }


    static AspArguments parse(Scanner s) {
	enterParser("arguments");

	AspArguments aa = new AspArguments(s.curLineNum());
	skip(s, leftParToken);
	if (s.curToken().kind != rightParToken) {
	    while (true) {
		aa.args.add(AspExpr.parse(s));
		if (s.curToken().kind == rightParToken) break;
		skip(s, commaToken);
	    }
	}
	skip(s, rightParToken);

	leaveParser("arguments");
	return aa;
    }


    @Override
    void prettyPrint() {
	prettyWrite("(");
	for (int i = 0;  i < args.size();  ++i) {
	    if (i > 0) prettyWrite(", ");
	    args.get(i).prettyPrint();
	}
	prettyWrite(")");
    }


    @Override
    RuntimeValue eval(RuntimeScope curScope) throws RuntimeReturnValue {
	RuntimeListValue aList = new RuntimeListValue();
	for (AspExpr ae: args) {
	    aList.addElem(ae.eval(curScope));
	}
	return aList;
    }
}
