// © 2021 Dag Langmyhr, Institutt for informatikk, Universitetet i Oslo

package no.uio.ifi.asp.parser;    

import java.util.ArrayList;

import no.uio.ifi.asp.main.*;
import no.uio.ifi.asp.runtime.*;
import no.uio.ifi.asp.scanner.*;
import static no.uio.ifi.asp.scanner.TokenKind.*;

class AspAssignment extends AspSmallStmt {
    AspName name;
    ArrayList<AspSubscription> indices = new ArrayList<>();
    AspExpr rhs;   // "rhs" = "Right-Hand Side"

    AspAssignment(int n) {
	super(n);
    }


    static AspAssignment parse(Scanner s) {
	enterParser("assignment");

	AspAssignment aa = new AspAssignment(s.curLineNum());
	aa.name = AspName.parse(s);
	while (s.curToken().kind == leftBracketToken) {
	    aa.indices.add(AspSubscription.parse(s));
	}
	skip(s, equalToken);
	aa.rhs = AspExpr.parse(s);

	leaveParser("assignment");
	return aa;
    }


    @Override
    void prettyPrint() {
	name.prettyPrint();
	for (AspSubscription as: indices) as.prettyPrint();
	prettyWrite(" = ");
	rhs.prettyPrint();
    }


    @Override
    RuntimeValue eval(RuntimeScope curScope) throws RuntimeReturnValue {
	String traceInfo = name.id;
	RuntimeValue rVal = rhs.eval(curScope);

	if (indices.isEmpty()) {
	    curScope.assign(name.id, rVal);
	} else {
	    RuntimeValue v = name.eval(curScope);
	    for (int i = 0;  i < indices.size()-1;  ++i) {
		RuntimeValue ix = indices.get(i).eval(curScope);
		RuntimeValue v2 = v.evalSubscription(ix, this);
		traceInfo += "[" + ix.showInfo() + "]";
		v = v2;
	    }
	    RuntimeValue lastInx = indices.get(indices.size()-1).eval(curScope);
	    traceInfo += "[" + lastInx.showInfo() + "]";
	    v.evalAssignElem(lastInx, rVal, this);
	}
	trace(traceInfo + " = " + rVal.showInfo());
	return null;
    }
}
