// © 2021 Dag Langmyhr, Institutt for informatikk, Universitetet i Oslo

package no.uio.ifi.asp.parser;    

import java.util.ArrayList;

import no.uio.ifi.asp.main.*;
import no.uio.ifi.asp.runtime.*;
import no.uio.ifi.asp.scanner.*;
import static no.uio.ifi.asp.scanner.TokenKind.*;

class AspComparison extends AspSyntax {
    ArrayList<AspTerm> terms = new ArrayList<>();
    ArrayList<AspCompOpr> oprs = new ArrayList<>();

    AspComparison(int n) {
	super(n);
    }


    static AspComparison parse(Scanner s) {
	enterParser("comparison");

	AspComparison ac = new AspComparison(s.curLineNum());
	ac.terms.add(AspTerm.parse(s));
	while (s.isCompOpr()) {
	    ac.oprs.add(AspCompOpr.parse(s));
	    ac.terms.add(AspTerm.parse(s));
	}

	leaveParser("comparison");
	return ac;
    }


    @Override
    void prettyPrint() {
	for (int i = 0;  i < terms.size();  ++i) {
	    terms.get(i).prettyPrint();
	    if (i < oprs.size())
		oprs.get(i).prettyPrint();
	}
    }


    @Override
    RuntimeValue eval(RuntimeScope curScope) throws RuntimeReturnValue {
	RuntimeValue v = terms.get(0).eval(curScope);
	RuntimeValue res = v;
	for (int i = 1;  i < terms.size();  ++i) {
	    RuntimeValue v2 = terms.get(i).eval(curScope);
	    TokenKind k = oprs.get(i-1).kind;
	    switch (k) {
	    case doubleEqualToken:
		res = v.evalEqual(v2, this);  break;
	    case greaterToken:
		res = v.evalGreater(v2, this);  break;
	    case greaterEqualToken:
		res = v.evalGreaterEqual(v2, this);  break;
	    case lessToken:
		res = v.evalLess(v2, this);  break;
	    case lessEqualToken:
		res = v.evalLessEqual(v2, this);  break;
	    case notEqualToken:
		res = v.evalNotEqual(v2, this);  break;
	    default:
		Main.panic("Illegal comparison operator: " + k + "!");
	    }
	    if (! res.getBoolValue("comparison operand",this)) return res;
	    v = v2;
	}
	return res;
    }
}
