// © 2021 Dag Langmyhr, Institutt for informatikk, Universitetet i Oslo

package no.uio.ifi.asp.parser;    

import java.util.ArrayList;

import no.uio.ifi.asp.main.*;
import no.uio.ifi.asp.runtime.*;
import no.uio.ifi.asp.scanner.*;
import static no.uio.ifi.asp.scanner.TokenKind.*;

class AspFactor extends AspSyntax {
    ArrayList<AspFactorOpr> oprs = new ArrayList<>();
    ArrayList<AspFactorPrefix> prefixes = new ArrayList<>();
    ArrayList<AspPrimary> primaries = new ArrayList<>();

    AspFactor(int n) {
	super(n);
    }


    static AspFactor parse(Scanner s) {
	enterParser("factor");

	AspFactor af = new AspFactor(s.curLineNum());
	while (true) {
	    if (s.isFactorPrefix()) {
		af.prefixes.add(AspFactorPrefix.parse(s));
	    } else {
		af.prefixes.add(null);
	    }

	    af.primaries.add(AspPrimary.parse(s));

	    if (! s.isFactorOpr()) break;
	    af.oprs.add(AspFactorOpr.parse(s));
	}

	leaveParser("factor");
	return af;
    }


    @Override
    void prettyPrint() {
	for (int i = 0;  i < primaries.size();  ++i) {
	    if (prefixes.get(i) != null)
		prefixes.get(i).prettyPrint();
	    primaries.get(i).prettyPrint();
	    if (i < oprs.size())
		oprs.get(i).prettyPrint();
	}
    }


    @Override
    RuntimeValue eval(RuntimeScope curScope) throws RuntimeReturnValue {
	RuntimeValue v = null;
	for (int i = 0;  i < primaries.size();  ++i) {
	    RuntimeValue vx = primaries.get(i).eval(curScope);

	    if (prefixes.get(i) != null) {
		switch (prefixes.get(i).kind) {
		case minusToken:
		    vx = vx.evalNegate(this);  break;
		case plusToken:
		    vx = vx.evalPositive(this);  break;
		default:
		    Main.panic("Illegal prefix operator: " + prefixes.get(i).kind + "!");
		}
	    }

	    if (i == 0) {
		v = vx;
	    } else {
		switch (oprs.get(i-1).kind) {
		case astToken:
		    v = v.evalMultiply(vx, this);  break;
		case doubleSlashToken:
		    v = v.evalIntDivide(vx, this);  break;
		case percentToken:
		    v = v.evalModulo(vx, this);  break;
		case slashToken:
		    v = v.evalDivide(vx, this);  break;
		default:
		    Main.panic("Illegal factor operator: " + oprs.get(i-1).kind + "!");
		}
	    }
	}
	return v;
    }
}
