// © 2021 Dag Langmyhr, Institutt for informatikk, Universitetet i Oslo

package no.uio.ifi.asp.parser;    

import java.util.ArrayList;

import no.uio.ifi.asp.main.*;
import no.uio.ifi.asp.runtime.*;
import no.uio.ifi.asp.scanner.*;
import static no.uio.ifi.asp.scanner.TokenKind.*;

class AspForStmt extends AspCompoundStmt {
    AspName id;
    AspExpr range;
    AspSuite body;

    AspForStmt(int n) {
	super(n);
    }


    static AspForStmt parse(Scanner s) {
	enterParser("for stmt");

	AspForStmt afs = new AspForStmt(s.curLineNum());
	skip(s, forToken);    afs.id = AspName.parse(s);  
	skip(s, inToken);     afs.range = AspExpr.parse(s);
	skip(s, colonToken);  afs.body = AspSuite.parse(s);

	leaveParser("for stmt");
	return afs;
    }


    @Override
    void prettyPrint() {
	prettyWrite("for ");  id.prettyPrint();  prettyWrite(" in ");
	range.prettyPrint();  prettyWrite(": ");
	body.prettyPrint();
    }


    @Override
    RuntimeValue eval(RuntimeScope curScope) throws RuntimeReturnValue {
	RuntimeValue r = range.eval(curScope);
	if (!(r instanceof RuntimeListValue))
	    RuntimeValue.runtimeError("For loop range is not a list!", this);
	RuntimeListValue rl = (RuntimeListValue)r;

	int i = 0;
	for (RuntimeValue rx: rl.listValue) {
	    curScope.assign(id.id, rx);
	    trace("for #" + (++i) + ": " + id.id + " = " + rx.showInfo());
	    body.eval(curScope);
	}
	return null;
    }
}
