// © 2021 Dag Langmyhr, Institutt for informatikk, Universitetet i Oslo

package no.uio.ifi.asp.parser;    

import java.util.ArrayList;

import no.uio.ifi.asp.main.*;
import no.uio.ifi.asp.runtime.*;
import no.uio.ifi.asp.scanner.*;
import static no.uio.ifi.asp.scanner.TokenKind.*;

public class AspFuncDef extends AspCompoundStmt {
    public ArrayList<AspName> formalParams = new ArrayList<>();
    public AspName fName;
    public AspSuite body;

    AspFuncDef(int n) {
	super(n);
    }

    static AspFuncDef parse(Scanner s) {
	enterParser("func def");

	AspFuncDef afd = new AspFuncDef(s.curLineNum());
	ArrayList<String> fParList = new ArrayList<>();
	skip(s, defToken);  
	afd.fName = AspName.parse(s);

	skip(s, leftParToken);
	if (s.curToken().kind != rightParToken) {
	    while (true) {
		test(s, nameToken);
		AspName pName = AspName.parse(s);
		if (fParList.contains(pName.id))
		    parserError("Parameter name " + pName.id + " defined twice!", 
				pName.lineNum);
		afd.formalParams.add(pName);  fParList.add(pName.id);
		if (s.curToken().kind == rightParToken) break;
		skip(s, commaToken);
	    }
	}
	skip(s, rightParToken);  skip(s, colonToken);
	afd.body = AspSuite.parse(s);

	leaveParser("func def");
	return afd;
    }


    @Override
    void prettyPrint() {
	prettyWrite("def ");  fName.prettyPrint();  prettyWrite(" (");
	for (int i = 0;  i < formalParams.size();  ++i) {
	    if (i > 0) prettyWrite(", ");
	    formalParams.get(i).prettyPrint();
	}
	prettyWrite("): ");
	body.prettyPrint();
	prettyWriteLn();
    }


    @Override
    RuntimeValue eval(RuntimeScope curScope) throws RuntimeReturnValue {
	curScope.assign(fName.id, new RuntimeFunc(fName.id,curScope,this));
	trace("def " + fName.id);
	return null;
    }
}
