// © 2021 Dag Langmyhr, Institutt for informatikk, Universitetet i Oslo

package no.uio.ifi.asp.parser;    

import java.util.ArrayList;

import no.uio.ifi.asp.main.*;
import no.uio.ifi.asp.runtime.*;
import no.uio.ifi.asp.scanner.*;
import static no.uio.ifi.asp.scanner.TokenKind.*;

class AspGlobalStmt extends AspSmallStmt {
    ArrayList<AspName> names = new ArrayList<>();;

    AspGlobalStmt(int n) {
	super(n);
    }

    static AspGlobalStmt parse(Scanner s) {
	enterParser("global stmt");

	AspGlobalStmt ags = new AspGlobalStmt(s.curLineNum());
	do {
	    s.readNextToken();
	    ags.names.add(AspName.parse(s));
	} while (s.curToken().kind == commaToken);

	leaveParser("global stmt");
	return ags;
    }

    @Override
    void prettyPrint() {
	prettyWrite("global ");
	int n = 0;
	for (AspName a: names) {
	    if (n++ > 0) prettyWrite(", ");
	    a.prettyPrint();
	}
    }

    @Override
    RuntimeValue eval(RuntimeScope curScope) throws RuntimeReturnValue {
	for (AspName a: names) {
	    if (curScope.hasDefined(a.id))
		RuntimeValue.runtimeError("Name " + a.id + 
	            " assigned to before global declaration!", this);
	    if (curScope.hasGlobalName(a.id))
		RuntimeValue.runtimeError("Name " + a.id + 
		    " already declared as global!", this);
	    curScope.registerGlobalName(a.id);
	}
	return new RuntimeNoneValue();
    }
}
