// © 2021 Dag Langmyhr, Institutt for informatikk, Universitetet i Oslo

package no.uio.ifi.asp.parser;    

import java.util.ArrayList;

import no.uio.ifi.asp.main.*;
import no.uio.ifi.asp.runtime.*;
import no.uio.ifi.asp.scanner.*;
import static no.uio.ifi.asp.scanner.TokenKind.*;

class AspIfStmt extends AspCompoundStmt {
    ArrayList<AspExpr> tests = new ArrayList<>();
    ArrayList<AspSuite> bodies = new ArrayList<>();
    AspSuite elsePart = null;

    AspIfStmt(int n) {
	super(n);
    }


    static AspIfStmt parse(Scanner s) {
	enterParser("if stmt");

	AspIfStmt ais = new AspIfStmt(s.curLineNum());
	do {
	    s.readNextToken();
	    ais.tests.add(AspExpr.parse(s));  skip(s, colonToken);
	    ais.bodies.add(AspSuite.parse(s));
	} while (s.curToken().kind == elifToken);

	if (s.curToken().kind == elseToken) {
	    skip(s, elseToken);  skip(s, colonToken);
	    ais.elsePart = AspSuite.parse(s);
	}

	leaveParser("if stmt");
	return ais;
    }


    @Override
    void prettyPrint() {
	for (int i = 0;  i < tests.size();  ++i) {
	    prettyWrite(i==0 ? "if " : "elif ");
	    tests.get(i).prettyPrint();  prettyWrite(": ");
	    bodies.get(i).prettyPrint();
	}
	if (elsePart != null) {
	    prettyWrite("else: ");
	    elsePart.prettyPrint();
	}
    }


    @Override
    RuntimeValue eval(RuntimeScope curScope) throws RuntimeReturnValue {
	for (int i = 0;  i < tests.size();  ++i) {
	    RuntimeValue t = tests.get(i).eval(curScope);
	    if (t.getBoolValue("if test",this)) {
		trace("if True alt #"+(i+1)+": ...");
		bodies.get(i).eval(curScope);
		return null;
	    }
	}
	if (elsePart != null) {
	    trace("else: ...");
	    elsePart.eval(curScope);
	}
	return null;
    }
}
