// © 2021 Dag Langmyhr, Institutt for informatikk, Universitetet i Oslo

package no.uio.ifi.asp.parser;    

import java.util.ArrayList;

import no.uio.ifi.asp.main.*;
import no.uio.ifi.asp.runtime.*;
import no.uio.ifi.asp.scanner.*;
import static no.uio.ifi.asp.scanner.TokenKind.*;

class AspPrimary extends AspSyntax {
    AspAtom atom;
    ArrayList<AspPrimarySuffix> suffices = new ArrayList<>();

    AspPrimary(int n) {
	super(n);
    }


    static AspPrimary parse(Scanner s) {
	enterParser("primary");

	AspPrimary ap = new AspPrimary(s.curLineNum());
	ap.atom = AspAtom.parse(s);
	while (s.curToken().kind==leftBracketToken || s.curToken().kind==leftParToken) {
	    ap.suffices.add(AspPrimarySuffix.parse(s));
	}

	leaveParser("primary");
	return ap;
    }


    @Override
    void prettyPrint() {
	atom.prettyPrint();
	for (AspPrimarySuffix aps: suffices)
	    aps.prettyPrint();
    }


    @Override
    RuntimeValue eval(RuntimeScope curScope) throws RuntimeReturnValue {
	RuntimeValue v = atom.eval(curScope);
	for (AspPrimarySuffix aps: suffices) {
	    if (aps instanceof AspSubscription) {
		v = v.evalSubscription(aps.eval(curScope), this);
	    } else {
		RuntimeListValue actParList = (RuntimeListValue)aps.eval(curScope);
		trace("Call "+v.showInfo()+" with params "+actParList.showInfo());
		v = v.evalFuncCall(actParList.listValue, this);
	    }
	}
	return v;
    }
}
