// © 2021 Dag Langmyhr, Institutt for informatikk, Universitetet i Oslo

package no.uio.ifi.asp.parser;    

import java.util.ArrayList;

import no.uio.ifi.asp.main.*;
import no.uio.ifi.asp.runtime.*;
import no.uio.ifi.asp.scanner.*;
import static no.uio.ifi.asp.scanner.TokenKind.*;

public class AspSmallStmtList extends AspStmt {
    ArrayList<AspSmallStmt> statements = new ArrayList<>();

    AspSmallStmtList(int n) {
	super(n);
    }


    static AspSmallStmtList parse(Scanner s) {
	enterParser("small stmt list");

	AspSmallStmtList assl = new AspSmallStmtList(s.curLineNum());
	while (true) {
	    assl.statements.add(AspSmallStmt.parse(s));
	    if (s.curToken().kind == newLineToken) break;
	    skip(s, semicolonToken);
	    if (s.curToken().kind == newLineToken) break;
	}
	skip(s, newLineToken);

	leaveParser("small stmt list");
	return assl;
    }


    @Override
    void prettyPrint() {
	int n = 0;
	for (AspSmallStmt ass: statements) {
	    if (n > 0) prettyWrite("; ");
	    ass.prettyPrint();  ++n;
	}
	prettyWriteLn();
    }


    @Override
    public RuntimeValue eval(RuntimeScope curScope) throws RuntimeReturnValue {
	for (AspSmallStmt ass: statements) {
	    ass.eval(curScope);
	}
	return null;
    }
}
