// © 2021 Dag Langmyhr, Institutt for informatikk, Universitetet i Oslo

package no.uio.ifi.asp.parser;    

import java.util.ArrayList;

import no.uio.ifi.asp.main.*;
import no.uio.ifi.asp.runtime.*;
import no.uio.ifi.asp.scanner.*;
import static no.uio.ifi.asp.scanner.TokenKind.*;

public class AspSuite extends AspSyntax {
    ArrayList<AspStmt> statements = new ArrayList<>();
    private boolean indentedSuite = false;

    AspSuite(int n) {
	super(n);
    }


    static AspSuite parse(Scanner s) {
	enterParser("suite");

	AspSuite as = new AspSuite(s.curLineNum());
	if (s.curToken().kind == newLineToken) {
	    as.indentedSuite = true;
	    skip(s, newLineToken);  
	    skip(s, indentToken);
	    do {
		as.statements.add(AspStmt.parse(s));
	    } while (s.curToken().kind != dedentToken);
	    skip(s, dedentToken);
	} else {
	    as.statements.add(AspSmallStmtList.parse(s));
	}

	leaveParser("suite");
	return as;
    }


    @Override
    void prettyPrint() {
	if (indentedSuite) {
	    prettyWriteLn();
	    prettyIndent();
	    for (AspStmt as: statements) 
		as.prettyPrint();
	    prettyDedent();
	} else {
	    statements.get(0).prettyPrint();
	}
    }


    @Override
    public RuntimeValue eval(RuntimeScope curScope) throws RuntimeReturnValue {
	for (AspStmt as: statements) {
	    as.eval(curScope);
	}
	return null;
    }
}
