// © 2021 Dag Langmyhr, Institutt for informatikk, Universitetet i Oslo

package no.uio.ifi.asp.parser;  

import java.util.ArrayList;

import no.uio.ifi.asp.main.*;
import no.uio.ifi.asp.runtime.*;
import no.uio.ifi.asp.scanner.*;
import static no.uio.ifi.asp.scanner.TokenKind.*;

class AspTerm extends AspSyntax {
    ArrayList<AspFactor> factors = new ArrayList<>();
    ArrayList<AspTermOpr> oprs = new ArrayList<>();

    AspTerm(int n) {
	super(n);
    }


    static AspTerm parse(Scanner s) {
	enterParser("term");

	AspTerm at = new AspTerm(s.curLineNum());
	at.factors.add(AspFactor.parse(s));
	while (s.isTermOpr()) {
	    at.oprs.add(AspTermOpr.parse(s));
	    at.factors.add(AspFactor.parse(s));
	}

	leaveParser("term");
	return at;
    }


    @Override
    void prettyPrint() {
	for (int i = 0;  i < factors.size();  ++i) {
	    factors.get(i).prettyPrint();
	    if (i < oprs.size()) 
		oprs.get(i).prettyPrint();
	}
    }


    @Override
    RuntimeValue eval(RuntimeScope curScope) throws RuntimeReturnValue {
	RuntimeValue v = factors.get(0).eval(curScope);
	for (int i = 1;  i < factors.size();  ++i) {
	    TokenKind k = oprs.get(i-1).kind;
	    switch (k) {
	    case minusToken:
		v = v.evalSubtract(factors.get(i).eval(curScope), this);  break;
	    case plusToken:
		v = v.evalAdd(factors.get(i).eval(curScope), this);  break;
	    default:
		Main.panic("Illegal term operator: " + k + "!");
	    }
	}
	return v;
    }
}
