// © 2021 Dag Langmyhr, Institutt for informatikk, Universitetet i Oslo

package no.uio.ifi.asp.runtime;    

import java.util.ArrayList;
import java.util.HashMap;

import no.uio.ifi.asp.main.*;
import no.uio.ifi.asp.parser.AspSyntax;

public class RuntimeDictValue extends RuntimeValue {
    HashMap<String,RuntimeValue> dictValue = new HashMap<>();


    @Override
    String typeName() {
	return "dictionary";
    }

    private String addQuotes(String s) {
	if (s.indexOf('\'') >= 0)
	    return '"' + s + '"';  
	return "'" + s + "'"; 
    }

    String showRecursiveInfo(ArrayList<RuntimeValue> inUse) {
	if (inUse.contains(this)) 
	    return "{...}";

	String im = "{";
	ArrayList<RuntimeValue> updatedUse = new ArrayList<>(inUse);
	updatedUse.add(this);

	for (String k: dictValue.keySet()) {
	    if (im.length() > 1) im += ", ";
	    RuntimeValue v = dictValue.get(k);
	    if (v instanceof RuntimeDictValue)
		im += addQuotes(k) + ": " + 
		    ((RuntimeDictValue)v).showRecursiveInfo(updatedUse);
	    else if (v instanceof RuntimeListValue)
		im += addQuotes(k) + ": " + 
		    ((RuntimeListValue)v).showRecursiveInfo(updatedUse);
	    else
		im += addQuotes(k) + ": " + v.showInfo();
	}
	return im + "}";
    }

    @Override
    public String toString() {
	return showRecursiveInfo(new ArrayList<RuntimeValue>());
    }


    public void setElem(String s, RuntimeValue v) {
	dictValue.put(s, v);
    }


    @Override
    public boolean getBoolValue(String what, AspSyntax where) {
	return (! dictValue.isEmpty());
    }


    @Override
    public void evalAssignElem(RuntimeValue inx, RuntimeValue val, AspSyntax where) {
	String inxVal = inx.getStringValue("index",where);
	setElem(inxVal, val);
    }


    @Override
    public RuntimeValue evalEqual(RuntimeValue v, AspSyntax where) {
	if (v instanceof RuntimeNoneValue) {
	    return new RuntimeBoolValue(false);
	}
	runtimeError("Type error for ==.", where);
	return null;  // Required by the compiler
    }


    @Override
    public RuntimeValue evalLen(AspSyntax where) {
	return new RuntimeIntValue(dictValue.size());
    }


    @Override
    public RuntimeValue evalNot(AspSyntax where) {
	return new RuntimeBoolValue(dictValue.isEmpty());
    }


    @Override
    public RuntimeValue evalNotEqual(RuntimeValue v, AspSyntax where) {
	if (v instanceof RuntimeNoneValue) {
	    return new RuntimeBoolValue(true);
	}
	runtimeError("Type error for !=.", where);
	return null;  // Required by the compiler
    }


    @Override
    public RuntimeValue evalSubscription(RuntimeValue v, AspSyntax where) {
	RuntimeValue res = null;

	if (v instanceof RuntimeStringValue) {
	    String v2 = v.getStringValue("dictionary key",where);
	    res = dictValue.get(v2);
	    if (res == null)
		runtimeError("Dictionary key " + v.showInfo() + " undefined!", where);
	} else {
	    runtimeError("A dictionary key must be a text string!", where);
	}
	return res;
    }
}
