// © 2021 Dag Langmyhr, Institutt for informatikk, Universitetet i Oslo

package no.uio.ifi.asp.runtime;    

import java.util.ArrayList;

import no.uio.ifi.asp.main.*;
import no.uio.ifi.asp.parser.AspFuncDef;
import no.uio.ifi.asp.parser.AspSyntax;

public class RuntimeFunc extends RuntimeValue {
    AspFuncDef def;
    RuntimeScope defScope;
    String name;

    public RuntimeFunc(String id, RuntimeScope ds, AspFuncDef f) {
	name = id;  def = f;  defScope = ds;
    }

    public RuntimeFunc(String id) {  // For library functions
	this(id, null, null);
    }


    @Override
    String typeName() {
	return "function";
    }


    @Override
    public String toString() {
	return "function " + name;
    }


    @Override
    public RuntimeValue evalFuncCall(ArrayList<RuntimeValue> actPars, 
	    AspSyntax where) {
	if (actPars.size() != def.formalParams.size()) 
	    runtimeError("Wrong number of parameters calling "+name+"!",where);
	RuntimeScope newScope = new RuntimeScope(defScope);
	for (int i = 0;  i < def.formalParams.size();  ++i) 
	    newScope.assign(def.formalParams.get(i).id, actPars.get(i));

	try {
	    def.body.eval(newScope);
	} catch (RuntimeReturnValue rrv) {
	    return rrv.value;
	}
	return new RuntimeNoneValue();
    }
}
