// © 2021 Dag Langmyhr, Institutt for informatikk, Universitetet i Oslo

package no.uio.ifi.asp.runtime;    

import java.util.ArrayList;

import no.uio.ifi.asp.main.*;
import no.uio.ifi.asp.parser.AspSyntax;

public class RuntimeListValue extends RuntimeValue {
    public ArrayList<RuntimeValue> listValue = new ArrayList<>();


    @Override
    String typeName() {
	return "list";
    }


    String showRecursiveInfo(ArrayList<RuntimeValue> inUse) {
	if (inUse.contains(this))
	    return "[...]";

	String im = "[";
	ArrayList<RuntimeValue> updatedUse = new ArrayList<>(inUse);
	updatedUse.add(this);

	for (int i = 0;  i < listValue.size();  ++i) {
	    if (i > 0) im += ", ";
	    RuntimeValue v = listValue.get(i);
	    if (v instanceof RuntimeDictValue)
		im += ((RuntimeDictValue)v).showRecursiveInfo(updatedUse);
	    else if (v instanceof RuntimeListValue)
		im += ((RuntimeListValue)v).showRecursiveInfo(updatedUse);
	    else
		im += v.showInfo();
	}
	return im + "]";
    }

    @Override
    public String toString() {
	return showRecursiveInfo(new ArrayList<RuntimeValue>());
    }


    public ArrayList<RuntimeValue> copyList() {
	return new ArrayList<RuntimeValue>(listValue);
    }


    public void addElem(RuntimeValue v) {
	listValue.add(v);
    }


    @Override
    public boolean getBoolValue(String what, AspSyntax where) {
	return (! listValue.isEmpty());
    }


    @Override
    public void evalAssignElem(RuntimeValue inx, RuntimeValue val, AspSyntax where) {
	long inxVal = inx.getIntValue("index",where);
	if (inxVal < 0 || inxVal >= listValue.size()) 
	    runtimeError("List index " + inxVal + " out of range!", where);

	listValue.set((int)inxVal, val);
    }


    @Override
    public RuntimeValue evalEqual(RuntimeValue v, AspSyntax where) {
	if (v instanceof RuntimeNoneValue) {
	    return new RuntimeBoolValue(false);
	}
	runtimeError("Type error for ==.", where);
	return null;  // Required by the compiler
    }


    @Override
    public RuntimeValue evalLen(AspSyntax where) {
	return new RuntimeIntValue(listValue.size());
    }


    @Override
    public RuntimeValue evalMultiply(RuntimeValue v, AspSyntax where) {
	RuntimeListValue res = null;

	if (v instanceof RuntimeIntValue) {
	    long v2 = v.getIntValue("* operand",where);
	    res = new RuntimeListValue();
	    for (long i = 1;  i <= v2;  ++i) 
		for (int j = 0;  j < listValue.size();  ++j)
		    res.addElem(listValue.get(j));
	} else { 
	    runtimeError("Type error for *.", where);
	}
	return res;
    }


    @Override
    public RuntimeValue evalNot(AspSyntax where) {
	return new RuntimeBoolValue(listValue.isEmpty());
    }


    @Override
    public RuntimeValue evalNotEqual(RuntimeValue v, AspSyntax where) {
	if (v instanceof RuntimeNoneValue) {
	    return new RuntimeBoolValue(true);
	}
	runtimeError("Type error for !=.", where);
	return null;  // Required by the compiler
    }


    @Override
    public RuntimeValue evalSubscription(RuntimeValue v, AspSyntax where) {
	if (v instanceof RuntimeIntValue) {
	    int v2 = (int)v.getIntValue("index",where);
	    if (v2 < 0 || v2 >= listValue.size())
		runtimeError("List index " + v2 + " out of range!", where);
	    return listValue.get(v2);
	}
	runtimeError("A list index must be an integer!", where);
	return null;  // Required by the compiler.
    }
}
