// © 2021 Dag Langmyhr, Institutt for informatikk, Universitetet i Oslo

package no.uio.ifi.asp.runtime;    

import no.uio.ifi.asp.main.*;
import no.uio.ifi.asp.parser.AspSyntax;

public class RuntimeStringValue extends RuntimeValue {
    String strValue;

    public RuntimeStringValue(String v) {
	strValue = v;
    }


    @Override
    String typeName() {
	return "string";
    }


    @Override
    public String showInfo() {
	if (strValue.indexOf('\'') >= 0)
	    return '"' + strValue + '"';  
	else
	    return "'" + strValue + "'"; 
    }


    @Override
    public String toString() {
	return strValue;
    }


    @Override
    public String getStringValue(String what, AspSyntax where) {
	return strValue;
    }


    @Override
    public boolean getBoolValue(String what, AspSyntax where) {
	return (strValue.length() > 0);
    }


    @Override
    public RuntimeValue evalAdd(RuntimeValue v, AspSyntax where) {
	if (v instanceof RuntimeStringValue) {
	    return new RuntimeStringValue(strValue + 
					  v.getStringValue("+ operand",where));
	}
	runtimeError("Type error for +.", where);
	return null;  // Required by the compiler.
    }


    @Override
    public RuntimeValue evalEqual(RuntimeValue v, AspSyntax where) {
	if (v instanceof RuntimeStringValue) {
	    String v2 = v.getStringValue("== operand",where);
	    return new RuntimeBoolValue(strValue.compareTo(v2)==0);
	} 
	if (v instanceof RuntimeNoneValue) {
	    return new RuntimeBoolValue(false);
	}
	runtimeError("Type error for ==.", where);
	return null;  // Required by the compiler.
    }


    @Override
    public RuntimeValue evalGreater(RuntimeValue v, AspSyntax where) {
	if (v instanceof RuntimeStringValue) {
	    String v2 = v.getStringValue("> operand",where);
	    return new RuntimeBoolValue(strValue.compareTo(v2) > 0);
	}
	runtimeError("Type error for >.", where);
	return null;  // Required by the compiler.
    }


    @Override
    public RuntimeValue evalGreaterEqual(RuntimeValue v, AspSyntax where) {
	if (v instanceof RuntimeStringValue) {
	    String v2 = v.getStringValue(">= operand",where);
	    return new RuntimeBoolValue(strValue.compareTo(v2) >= 0);
	}
	runtimeError("Type error for >=.", where);
	return null;  // Required by the compiler.
    }


    @Override
    public RuntimeValue evalLen(AspSyntax where) {
	return new RuntimeIntValue(strValue.length());
    }


    @Override
    public RuntimeValue evalLess(RuntimeValue v, AspSyntax where) {
	if (v instanceof RuntimeStringValue) {
	    String v2 = v.getStringValue("< operand",where);
	    return new RuntimeBoolValue(strValue.compareTo(v2)<0);
	} 
	runtimeError("Type error for <.", where);
	return null;  // Required by the compiler.
    }


    @Override
    public RuntimeValue evalLessEqual(RuntimeValue v, AspSyntax where) {
	if (v instanceof RuntimeStringValue) {
	    String v2 = v.getStringValue("<= operand",where);
	    return new RuntimeBoolValue(strValue.compareTo(v2)<=0);
	}
	runtimeError("Type error for <=.", where);
	return null;  // Required by the compiler.
    }


    @Override
    public RuntimeValue evalMultiply(RuntimeValue v, AspSyntax where) {
	if (v instanceof RuntimeIntValue) {
	    long v2 = v.getIntValue("* operand",where);
	    String s = "";
	    for (long i = 1;  i <= v2;  ++i)
		s += strValue;
	    return new RuntimeStringValue(s);
	}
	runtimeError("Type error for *.", where);
	return null;  // Required by the compiler.
    }


    @Override
    public RuntimeValue evalNot(AspSyntax where) {
	return new RuntimeBoolValue(strValue.length() == 0);
    }


    @Override
    public RuntimeValue evalNotEqual(RuntimeValue v, AspSyntax where) {
	if (v instanceof RuntimeStringValue) {
	    String v2 = v.getStringValue("!= operand",where);
	    return new RuntimeBoolValue(strValue.compareTo(v2) != 0);
	} 
	if (v instanceof RuntimeNoneValue) {
	    return new RuntimeBoolValue(true);
	} 
	runtimeError("Type error for !=.", where);
	return null;  // Rquired by the compiler.
    }


    @Override
    public RuntimeValue evalSubscription(RuntimeValue v, AspSyntax where) {
	if (v instanceof RuntimeIntValue) {
	    int v2 = (int)v.getIntValue("index",where);
	    if (v2 < 0 || v2 >= strValue.length())
		runtimeError("String index " + v2 + " out of range!", where);
	    return new RuntimeStringValue(strValue.substring(v2,v2+1));
	} 
	runtimeError("A string index must be an integer!", where);
	return null;  // Required by the compiler.
    }
}
